<?php
/*
Customize this class in order to bind FileRun to a third party authentication source.
*/

class customAuth {
	/*
	This method can be blank.
	Useful for initializing variables.
	*/
	function customAuth() {
		$this->secretKey = "passwd123";
		$this->customSession = true;
		//init vars
		$this->config['servers']['A'] = array(
			"host" => "ldap.openldap.org",
			"port" => false, //default
			"base_dn" => "dc=openldap,dc=org",
			"filter" => "(&(objectclass=user)(sAMAccountName={USERNAME}))",
			"domain_suffix" => "",
			"ldap_protocol_version" => 3,
			"ldap_referrals" => 0,
			"fields_mapping" => array(
				"ldap_username_field" => "samaccountname",
				"ldap_firstname_field" => "displayName",
				"ldap_lastname_field" => "sn",
				"ldap_company_field" => "department",
				"ldap_email_field" => "userprincipalname"
			)
		);
		if (!$_SESSION['authServer']) {
			$_SESSION['authServer'] = "A";
		} else {
			$_SESSION['authServer'] = fromHTML($_SESSION['authServer']);
		}
		$this->currentServer = $this->config['servers'][$_SESSION['authServer']];
	}
	
	/*
	Get authentication session from thirdparty software and apply it locally
	*/
	function setSessionCredentials() {
	}
	
	/*
	This method is called on each FileRun request
	*/
	function authenticate($username, $password, $doLogin) {
		if (!function_exists('ldap_connect')) {
			echo "PHP is missing LDAP support.";
			exit();
		}
		$this->_connectionID = ldap_connect($this->currentServer['host'], $this->currentServer['port']);
		if (!$this->_connectionID) {
			echo "Failed to connect to LDAP server!";
			exit();
		}
		
		if ($this->currentServer['ldap_protocol_version'] !== false) {
			ldap_set_option($this->_connectionID, LDAP_OPT_PROTOCOL_VERSION, $this->currentServer['ldap_protocol_version']);
		}
		if ($this->currentServer['ldap_referrals'] !== false) {
			ldap_set_option($this->_connectionID, LDAP_OPT_REFERRALS, $this->currentServer['ldap_referrals']); 
		}
		
		if (!$doLogin) {
			$password = $_SESSION['FileRun']['password'];
			$password = cryptdecrypt($password, $this->secretKey);//decrypt session password to be used against LDAP
		} else {
			//$password = "test";
			$crypt = cryptdecrypt($password, $this->secretKey);//encrypt the password for storing into session
		}
		
		$rs = @ldap_bind($this->_connectionID, $username.$this->currentServer['domain_suffix'], $password);
		if (!$rs) {
			echo "{rs:false, error:'Failed to login!'}";
			exit();
		} else {
			$filter = str_replace("{USERNAME}", $username, $this->currentServer['filter']);
			$rs = ldap_search($this->_connectionID, $this->currentServer['base_dn'], $filter);
			if (!$rs) {
				echo "Failed to retrieve LDAP user information! (".ldap_error($this->_connectionID).")";
				exit();
			}
			$entries = ldap_get_entries($this->_connectionID, $rs);
			
			if ($entries['count'] == 0) {
				echo "LDAP user information not found!";
				exit();
			} else {
				$row = $entries[0];
				$m = $this->currentServer['fields_mapping'];
				$ldapUserInfo = array(
					"username" => $row[$m['ldap_username_field']][0],
					"firstname" => $row[$m['ldap_firstname_field']][0] ? $row[$m['ldap_firstname_field']][0] : "-",
					"lastname" => $row[$m['ldap_lastname_field']][0] ? $row[$m['ldap_lastname_field']][0] : "-",
					"company" => $row[$m['ldap_company_field']][0] ? $row[$m['ldap_company_field']][0] : "-",
					"email" => $row[$m['ldap_email_field']][0] ? $row[$m['ldap_email_field']][0] : "-"
				);
				
				if (!$ldapUserInfo['username']) {
					echo "Error: Missing LDAP username information!";
					exit();
				} else {//user logged in and record found
					
					$success = true;
					//set user's data for inserting into FileRun MySQL database
					//note: the user will be added only once, if not found in local database
					$userData = array(
					    "username" => $username,
					    "password" => "",
						"cookie" => "",
						"owner" => "",	//id of independent admin user
						"registration_date" => time(),
					    "activated" => 1,
						"failed_login_attempts" => 0,
					    "last_login_date" => time(),
					    "email" => $ldapUserInfo['email'],
					    "receive_notifications" => 0,
						"new_email" => "",
					    "name" => $ldapUserInfo["firstname"].($ldapUserInfo["lastname"]?" ".$ldapUserInfo["lastname"]:""),
					    "company" => $ldapUserInfo['company'],
					    "website" => "",
						"description" => "",
						"logo_url" => ""
					);
					$users_home_folder_template = "C:/FileRun/system/data/default_home_folder/{USERNAME}";
					$userIsAdmin = false;
					$userPerms = array(
						"role" => "", //role id
						"admin_type" => ($userIsAdmin ? "simple" : ""), //"indep"
						"admin_users" => $userIsAdmin ? 1 : 0,
						"admin_roles" => $userIsAdmin ? 1 : 0,
						"admin_notifications" => $userIsAdmin ? 1 : 0,
						"admin_logs" => $userIsAdmin ? 1 : 0,
						"admin_metadata" => $userIsAdmin ? 1 : 0,
						"admin_over" => 0,
						"admin_max_users" => 0,
						"admin_homefolder_template" => "",
						"homefolder" => str_replace("{USERNAME}", $username, $users_home_folder_template),
						"space_quota_max" => 0,
						"space_quota_current" => 0,
						"traffic_quota_max" => 0,
						"traffic_quota_current" => 0,
						"readonly" => 0,
						"upload" => 1,
						"download" => 1,
						"email" => 1,
						"weblink" => 1,
						"share" => 1,
						"users_may_see" => "-ALL-"
					);
					if ($userPerms['homefolder'] && !is_dir($userPerms['homefolder'])) {//create home folder
						@mkdir($userPerms['homefolder']);
					}
				}
			}
		}
		
		//do not edit this part
		if ($success) {//credentials checked
			$rs = array($userData, $userPerms);
			$rs['sessionPassword'] = $crypt;
		    return $rs;
		} else {
			return false;
		}
	}
}

function cryptdecrypt($text, $key = '') {
    // return text unaltered if the key is blank
    if ($key == '') {
        return $text;
    }
    // remove the spaces in the key
    $key = str_replace(' ', '', $key);
    if (strlen($key) < 8) {
        exit('key error');
    }
    // set key length to be no more than 32 characters
    $key_len = strlen($key);
    if ($key_len > 32) {
        $key_len = 32;
    }
    // A wee bit of tidying in case the key was too long
    $key = substr($key, 0, $key_len);
    // We use this a couple of times or so
    $text_len = strlen($text);
    // fill key with the bitwise AND of the ith key character and 0x1F, padded to length of text.
    $lomask = str_repeat("\x1f", $text_len); // Probably better than str_pad
    $himask = str_repeat("\xe0", $text_len);
    $k = str_pad("", $text_len, $key); // this one _does_ need to be str_pad
    // {en|de}cryption algorithm
    $text = (($text ^ $k) & $lomask) | ($text & $himask);
    return $text;
} 
?>